//////////////////////////////////////////////////////////////////////////////////////////
//											//
//	Filename 		: 		Simulator.cpp				//
//											//	
//	Author			:		Raman Dhillon				//
//											//
//	Version			: 		1.0					//
//											//
//	Location		:		Pervasive Communication Lab 		//
//						University of Colorado at Boulder	//
//											//
//	Change Log									//
//	--------------------------------------------------------------------------	//
//	Change		Author		Date 		Reason				//
//											//
//	--------------------------------------------------------------------------	//
//											//
//////////////////////////////////////////////////////////////////////////////////////////


#include "Simulator.h"
#include "ConfigReader.h"
#include "TrafficInfo.h"
#include <pthread.h>
#include <signal.h>
#include <getopt.h>

int noOfUsers =0;
double hurstParam =0.0 ;
double dataRatePerUser =0.0;
double meanFileSize =0.0;
double stdDevFileSize = -1;
string serverAddr = "";
string configFile = "config.xml";
string scenarioNumber = "SCENARIO1";

int startingSeed = 0;
int simDuration = -1;


TrafficInfo* trafficInfo = NULL;
Simulator* simulator = NULL;

void help();

void* handleCtrlC(void *arg)
{
	sigset_t signal_set;
        int sig;

	for(;;)	
	{

		sigfillset(&signal_set);
        	sigwait( &signal_set, &sig );

		if(sig == SIGINT || sig == SIGALRM)
		{

        		cout << "\n--------------------------------------------------------------------------------------------------------------------";  
        		cout << "\n\n\n--------------------------------------------------------------------------------------------------------------------";  
			cout << "\nEnding simulation";
			if(simulator) 
				simulator->stopSimulation();

			cout << "\nSimulation ended";
        		cout << "\n--------------------------------------------------------------------------------------------------------------------";  

        		cout << "\n\n\n--------------------------------------------------------------------------------------------------------------------";  
	     		cout << "\n                                   		USER STATISTICS";
        		cout << "\n--------------------------------------------------------------------------------------------------------------------";  

                	cout << "\nUser   " << setw(5) << "Reqs" << "   " << setw(5) << "Resp" << "   " << setw(9) << "Data"<< "    " << setw(12) << "Rate" << "      " << setw(10) << "MinFile" << "      " << setw(10) << "MaxFile" << "      " << setw(10) << "MinArr" << "     " << setw(10) << "MaxArr";
        		cout << "\n--------------------------------------------------------------------------------------------------------------------";  

			break;
		}

	}
}


int main(int argc, char* argv[])
{

 sigset_t signal_set;
 pthread_t sig_thread;

  int optch;
  static char optstring[] = "c:s:n:h:r:m:d:a:t:";
  opterr=0;

  	if(argc > 1)
        {

                do
                {
                        opterr = 0;
                        optch = getopt(argc, argv, optstring);

			switch(optch)
                        {
                        case 'c':
                                configFile.assign(optarg);

                                break;
                        case 's':
                                scenarioNumber = "SCENARIO";
                                scenarioNumber += optarg;
                                break;
                        case 'n':
                                noOfUsers = atoi(optarg);
                                break;

                        case 'h':
                                hurstParam = atof(optarg);
                                break;

                        case 'r':
                                dataRatePerUser = atof(optarg);
                                break;

                        case 'm':
                                meanFileSize = atof(optarg);
                                break;
                        case 'd':
                                stdDevFileSize = atof(optarg);
                                break;
                        case 'a':
                                serverAddr.assign(optarg);
                                break;
			case 't': 
				simDuration = atoi(optarg);
				break;
                        case '?':
 				help();
                                exit(1);
                        }

                } while ((optch != '?') && (optch != -1));

	}

  sigfillset(&signal_set); 
  pthread_sigmask( SIG_BLOCK, &signal_set, NULL);

  pthread_create( &sig_thread, NULL, handleCtrlC, NULL);

  
  ConfigReader* reader = ConfigReader::getConfigInstance((char*)configFile.c_str());

  if(reader->initializeXMLReader()!=0)
  {	
 	cout << "\nError in parsing Configuration file.\n";
 
 	if(reader)
  	{ 
  		delete reader;
  		reader = NULL;
  	}

	return -1;	
  }

  trafficInfo = reader->buildTrafficScenario((char*)scenarioNumber.c_str());
   
  if(trafficInfo==NULL)
  {
	cout << "\nError: Invalid scenario.\n";
	
 	if(reader)
  	{ 
  		delete reader;
  		reader = NULL;
  	}

	return -1;	

  }
 
 if(noOfUsers != 0) trafficInfo->setNoOfUsers(noOfUsers); 
 if(hurstParam != 0) trafficInfo->setHurstParameter(hurstParam);
 if(dataRatePerUser != 0) trafficInfo->setDataRatePerUser(dataRatePerUser);
 if(meanFileSize != 0) trafficInfo->setMeanFileSize(meanFileSize);
 if(stdDevFileSize >= 0) trafficInfo->setStdDevFileSize(stdDevFileSize);
 if(serverAddr != "") trafficInfo->setServerURL(serverAddr);

 if(simDuration < 0) 
 {	
	simDuration = trafficInfo->getSimulationDuration();
 }
  simulator = new Simulator(trafficInfo);

  if(simDuration != 0)
  	alarm(simDuration);

  simulator->simulateUsers();

  if(simulator)
  {
  	delete simulator;
  	simulator = NULL;
  }

  if(reader)
  { 
  	delete reader;
  	reader = NULL;
  }

  pthread_cancel(sig_thread);

  return 0;
}

void help()
{
        cout << "\nUsage:";

        cout << "\n   ./Simulator <option(s)>";

        cout << "\n\nRun Simulator without any options for default parameters, i.e. config.xml Scenario 1.";

        cout << "\nThe following options can be supplied to the command:\n";

        cout << "\n-c config    - read from the specified configuration file";
        cout << "\n-s scenar    - scenario number to run, must be present in the configuration file";
        cout << "\n-n nuser     - specifies the number of users to be simulated";
        cout << "\n-h hurst     - hurst parameter to be used";
        cout << "\n-r drate     - data rate per simulated user";
        cout << "\n-m mfsize    - mean file size for web responses";
        cout << "\n-d stddev    - std dev of the file size distribution";
        cout << "\n-a srvaddr   - address of the server serving web requests";
	cout << "\n-t timedur   - time duration of the simulation";
        cout << "\n\nSimulator 1.0, Pecolab, University of Colorado at Boulder";
	cout << "\nNOTE: This tool is based on GNU libraries and tools.\n\n"; 

}


