//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               TrafficInfo.h                           //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef _TRAFFIC_INFO_H
#define _TRAFFIC_INFO_H

#include <iostream>
#include <string>

using namespace std;

#define HTTP_TRAFFIC 0
#define KBYTES 0
#define MBYTES 1
#define GBYTES 2


class TrafficInfo
{

	int noOfUsers;
	int trafficType;

	double dataRatePerUser;		// in kbps
	double hurstParameter;

	double paretoExp;
	double meanInterArrTime;
	double paretoMinValue;

	double meanFileSize;			// in kB
	double stdDevFileSize;
	int fileSizeUnits;

	int startingSeed;
	int simDuration;

	string serverIPAddr;

    public:

	TrafficInfo(int users , double hparam, double dataRate, string ipAddr)
	: noOfUsers(users), hurstParameter(hparam), dataRatePerUser(dataRate), serverIPAddr(ipAddr) 
	{
		paretoExp = 3 - 2 * (hurstParameter);
		paretoMinValue = 1.0;

		meanFileSize = 5.0;
		stdDevFileSize = 2.0; 
		fileSizeUnits = KBYTES;

		simDuration = 0;
		startingSeed = 0;

		trafficType = HTTP_TRAFFIC;
	}

	TrafficInfo(int users , double hParam, double dataRate, double fileSize, double devFileSize, int units,
	string ipAddr, int duration, int seed)
	: noOfUsers(users), hurstParameter(hParam), dataRatePerUser(dataRate), meanFileSize(fileSize),
	  stdDevFileSize(devFileSize), fileSizeUnits(units), simDuration(duration), startingSeed(seed)
	{
		paretoExp = 3 - 2 * (hurstParameter);


		if(fileSizeUnits == KBYTES)
			meanInterArrTime = (meanFileSize * 8) / dataRatePerUser;

		else if(fileSizeUnits == MBYTES)
			meanInterArrTime = (meanFileSize * 8 * 1000) / dataRatePerUser;
			
		else if(fileSizeUnits == GBYTES)
			meanInterArrTime = (meanFileSize * 8 * 1000 * 1000) / dataRatePerUser;

		paretoMinValue = meanInterArrTime * (paretoExp - 1) / paretoExp;

		trafficType = HTTP_TRAFFIC;
		serverIPAddr.assign(ipAddr);
	}

	~TrafficInfo(){}	

	int getNoOfUsers() { return noOfUsers; }
	void setNoOfUsers(int users) { noOfUsers = users; }

	int getTrafficType() { return trafficType;}
	void setTrafficType(int trafType) { trafficType = trafType;}

	double getDataRatePerUser() { return dataRatePerUser; }
	void setDataRatePerUser(double dataRate) 
	{ 
		dataRatePerUser = dataRate;

		if(fileSizeUnits == KBYTES)
			meanInterArrTime = (meanFileSize * 8) / dataRatePerUser;

		else if(fileSizeUnits == MBYTES)
			meanInterArrTime = (meanFileSize * 8 * 1000) / dataRatePerUser;
			
		else if(fileSizeUnits == GBYTES)
			meanInterArrTime = (meanFileSize * 8 * 1000 * 1000) / dataRatePerUser;

		paretoMinValue = meanInterArrTime * (paretoExp - 1) / paretoExp;
	} 

	double getHurstParameter() { return hurstParameter;}

	void setHurstParameter(double hparam) 
	{ 
		hurstParameter = hparam;

		paretoExp = 3 - 2 * (hurstParameter);
		paretoMinValue = meanInterArrTime * (paretoExp - 1) / paretoExp;
	}

	double getParetoExp() { return paretoExp; }
	double getParetoMinValue() { return paretoMinValue; }
	double getMeanInterArrTime() { return meanInterArrTime; }

	double getMeanFileSize() { return meanFileSize;}
	
	void setMeanFileSize(double fileSize) 
	{ 
		meanFileSize = fileSize;
	
		if(fileSizeUnits == KBYTES)
			meanInterArrTime = (meanFileSize * 8) / dataRatePerUser;

		else if(fileSizeUnits == MBYTES)
			meanInterArrTime = (meanFileSize * 8 * 1000) / dataRatePerUser;
			
		else if(fileSizeUnits == GBYTES)
			meanInterArrTime = (meanFileSize * 8 * 1000 * 1000) / dataRatePerUser;

		paretoMinValue = meanInterArrTime * (paretoExp - 1) / paretoExp;
	}

	double getStdDevFileSize() { return stdDevFileSize;}
	void setStdDevFileSize(double devFileSize) { stdDevFileSize = devFileSize;}

	int getFileSizeUnits() { return fileSizeUnits; }
	void setFileSizeUnits(int units) { fileSizeUnits = units; }

	string getServerURL() { return serverIPAddr; }
	void setServerURL(string ipAddr) { serverIPAddr = ipAddr; }

	int getSimulationDuration() { return simDuration; }
	
	void setStartingSeed(int seed) { startingSeed = seed; } 
	int getStartingSeed() { return startingSeed; } 


	void showTrafficChars()
	{
		cout << "\n--------------------------------------------------------------------------------------------------------------------";
		cout << "\nTraffic Characteristics";
		cout << "\n--------------------------------------------------------------------------------------------------------------------";

		string units;	

		switch(fileSizeUnits) 
		{
			case KBYTES: units = "kB"; break;
			case MBYTES: units = "MB"; break;
			case GBYTES: units = "GB"; break;
		} 
	
		cout << "\nNumber of Users = " << noOfUsers;
		cout << "\nHurst Parameter = " << hurstParameter;
		cout << "\nData Rate per User = " << dataRatePerUser << " kbps";
		cout << "\nMean File Size = " << meanFileSize << " " << units;
		cout << "\nStd Dev File Size = " << stdDevFileSize << " " << units;
		cout << "\nMean Inter-arrival Time = " << meanInterArrTime << " sec";
		cout << "\nPareto Exponent = " << paretoExp;
		cout << "\nPareto Minimum Value = " << paretoMinValue << " sec";
		cout << "\nServer address = " << serverIPAddr;
		cout << "\n--------------------------------------------------------------------------------------------------------------------";
	}

};

#endif
